package logic

import (
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"

	"context"

	"fusenapi/server/info/internal/svc"
	"fusenapi/server/info/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type AddressAddLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewAddressAddLogic(ctx context.Context, svcCtx *svc.ServiceContext) *AddressAddLogic {
	return &AddressAddLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *AddressAddLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

func (l *AddressAddLogic) AddressAdd(req *types.AddressRequest, userinfo *auth.UserInfo) (resp *basic.Response) {
	// 返回值必须调用Set重新返回, resp可以空指针调用 resp.SetStatus(basic.CodeOK, data)
	// userinfo 传入值时, 一定不为null

	if !userinfo.IsUser() {
		return resp.SetStatus(basic.CodeUnAuth)
	}

	// 确认这个IsDefault的值范围
	if !auth.CheckValueRange(req.IsDefault, 0, 1) {
		return resp.SetStatus(basic.CodeSafeValueRangeErr) // IsDefault值超出范围, 返回安全值范围错误
	}

	m := l.svcCtx.AllModels.FsAddress // 创建地址模型

	// 如果ID为0, 表示新增地址

	var (
		country string = "USA" // 国家默认为美国
		status  int64  = 1     // 默认地址状态为1(正常)
	)

	createOne := &gmodel.FsAddress{ // 构建FsAddress结构体
		FirstName: &req.FirstName,
		LastName:  &req.LastName,
		Mobile:    &req.Mobile,
		Street:    &req.Street,
		Suite:     &req.Suite,
		City:      &req.City,
		State:     &req.State,
		Country:   &country,
		Status:    &status,
		UserId:    &userinfo.UserId,
		ZipCode:   &req.ZipCode,
	}
	address, err := m.CreateOne(l.ctx, createOne) // 新增地址
	if err != nil {
		logx.Error(err)                              // 日志记录错误
		return resp.SetStatus(basic.CodeDbCreateErr) // 返回数据库创建错误
	}

	if req.IsDefault > 0 {
		m.SettingUserDefaultAddress(l.ctx, userinfo.UserId, address.AddressId)
	}

	addresses, err := m.GetUserAllAddress(l.ctx, userinfo.UserId)
	if err != nil {
		logx.Error(err)
		return resp.SetStatus(basic.CodeDbSqlErr) // 返回数据库创建错误
	}

	return resp.SetStatus(basic.CodeOK, map[string]any{
		"address_id":   address.AddressId,
		"address_list": addresses,
	}) // 返回成功并返回地址ID

}

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *AddressAddLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
